/*
 * bt-inspector: a small tool to inspect various bits into a video
 *
 * Copyright (C) 2010 Luc Saillard <luc@saillard.org>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */
#include <config.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <locale.h>
#include <glib/gi18n.h>
#include <glib.h>
#include "utils.h"

#include <libavdevice/avdevice.h>
#include <libavformat/avformat.h>

static int   opt_verbose;


static GOptionEntry options_entries[] =
{
  { "verbose", 'v', 0, G_OPTION_ARG_NONE, &opt_verbose, N_("Show more output on the console"), NULL},
  { NULL }
};


struct bt_stream
{
  int        index;
  AVRational time_base;
  GArray    *packets;

  /* computed value */
  uint64_t   duration;
  uint64_t   size;
};

struct bt_packet
{
  int duration;
  int size;

  /* computed value */
  int    bitrate;
};

struct video_stats
{
  struct bt_stream *audio;
  struct bt_stream *video;
};




static struct video_stats *parse_file(const char *filename);

static void stream_output_bitrate_data(struct bt_stream *stream, const char *output_filename);
static void output_bitrate_gnuplot(struct video_stats *stats, const char *prefix);


int main(int argc, char **argv)
{
  GOptionContext *option_context;
  GError *gerror = NULL;
  char *input_filename;
  char *output_prefix;
  struct video_stats *stats;

  setlocale(LC_ALL, "");

  bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
  bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
  textdomain(GETTEXT_PACKAGE);

  option_context = g_option_context_new(_("- bitrate inspector"));
  g_option_context_add_main_entries(option_context, options_entries, PACKAGE_NAME);
  g_option_context_parse(option_context, &argc, &argv, &gerror);

  /* register all codecs, demux and protocols */
  avcodec_register_all();
  avdevice_register_all();
  av_register_all();

  if (argc < 2)
   {
     const char *help = g_option_context_get_help(option_context, TRUE, NULL);
     error("An input file must be specified\n%s\n", help);
     return 1;
   }

  input_filename = argv[1];
  output_prefix = g_path_get_basename(input_filename);

  stats = parse_file(input_filename);
  if (stats == NULL)
    return 2;

  output_bitrate_gnuplot(stats, output_prefix);

  return 0;
}

static void
print_error(const char *filename, int err)
{
    switch(err) {
    case AVERROR_NUMEXPECTED:
        fprintf(stderr, "%s: Incorrect image filename syntax.\n"
                "Use '%%d' to specify the image number:\n"
                "  for img1.jpg, img2.jpg, ..., use 'img%%d.jpg';\n"
                "  for img001.jpg, img002.jpg, ..., use 'img%%03d.jpg'.\n",
                filename);
        break;
    case AVERROR_INVALIDDATA:
        fprintf(stderr, "%s: Error while parsing header\n", filename);
        break;
    case AVERROR_NOFMT:
        fprintf(stderr, "%s: Unknown format\n", filename);
        break;
    case AVERROR(EIO):
        fprintf(stderr, "%s: I/O error occurred\n"
                "Usually that means that input file is truncated and/or corrupted.\n",
                filename);
        break;
    case AVERROR(ENOMEM):
        fprintf(stderr, "%s: memory allocation error occurred\n", filename);
        break;
    case AVERROR(ENOENT):
        fprintf(stderr, "%s: no such file or directory\n", filename);
        break;
#if CONFIG_NETWORK
    case AVERROR(FF_NETERROR(EPROTONOSUPPORT)):
        fprintf(stderr, "%s: Unsupported network protocol\n", filename);
        break;
#endif
    default:
        fprintf(stderr, "%s: Error while opening file\n", filename);
        break;
    }
}


static void
add_packet(const struct AVPacket *pkt, struct bt_stream *stream)
{
  struct bt_packet info;

  info.duration = pkt->duration;
  info.size = pkt->size;
  info.bitrate = pkt->size / ((double)pkt->duration * stream->time_base.num / stream->time_base.den);

  stream->duration += pkt->duration;
  stream->size += pkt->size;

  g_array_append_val(stream->packets, info);

  if (0)
   {
     int i = stream->packets->len;
     uint64_t duration_in_us;
     double bitrate;
     duration_in_us = 1000000UL * pkt->duration * stream->time_base.num / stream->time_base.den;
     bitrate = pkt->size / ((double)pkt->duration * stream->time_base.num / stream->time_base.den);
     trace("[%4.4d] size=%s duration=%s  bitrate=%s\n", i,
	   size_to_string("", pkt->size),
	   time_us_to_string(duration_in_us),
	   network_size_to_string("", bitrate, "/s"));
   }
}




static struct video_stats *
parse_file(const char *filename)
{
  AVFormatContext *fmt = NULL;
  AVPacket pkt;
  int ret;
  unsigned int i;
  struct video_stats *stats;

  ret = av_open_input_file(&fmt, filename, NULL, 0, NULL);
  if (ret < 0)
   {
     print_error(filename, ret);
     return NULL;
   }

  ret = av_find_stream_info(fmt);
  if (ret < 0)
   {
     error("%s: could not find codec parameters\n", filename);
     av_close_input_file(fmt);
     return NULL;
   }

  if (opt_verbose)
    dump_format(fmt, 0, filename, 0);


  stats = g_new0(struct video_stats, 1);

  /* Read streams, and allocated memory */
  for (i=0; i<fmt->nb_streams; i++)
   {
     AVStream *st = fmt->streams[i];
     AVCodecContext *enc = st->codec;

     switch(enc->codec_type)
      {
	case CODEC_TYPE_AUDIO:
	  if (stats->audio)
	    break;

	  stats->audio = g_new0(struct bt_stream, 1);
	  stats->audio->packets = g_array_sized_new(FALSE, TRUE, sizeof(struct bt_packet), 300);
	  stats->audio->index = i;
	  stats->audio->time_base = st->time_base;
	  break;

	case CODEC_TYPE_VIDEO:
	  if (stats->video)
	    break;

	  stats->video = g_new0(struct bt_stream, 1);
	  stats->video->packets = g_array_sized_new(FALSE, TRUE, sizeof(struct bt_packet), 300);
	  stats->video->index = i;
	  stats->video->time_base = st->time_base;
	  break;

	default:
	  break;
      }
   }

  /* Read packets, and store revelant information info the stream struct */

  i = 0;
  while (1)
   {
     memset(&pkt, 0, sizeof(pkt));
     ret = av_read_frame(fmt, &pkt);
     if (ret < 0)
      {
	debug("No more frames\n");
	break;
      }

     if (pkt.stream_index == stats->audio->index)
       add_packet(&pkt, stats->audio);
     else if (pkt.stream_index == stats->video->index)
       add_packet(&pkt, stats->video);

     i++;
   }

  trace("success\n");

  return stats;
}

static double
stream_get_avg_bitrate(struct bt_stream *stream)
{
  return stream->size / ((double)stream->duration * stream->time_base.num / stream->time_base.den);
}


static void
stream_output_bitrate_data(struct bt_stream *stream, const char *output_filename)
{
  FILE *FP;
  struct bt_packet *p;
  unsigned int i;

  if (stream == NULL)
    return;

  FP = fopen(output_filename, "w");
  if (FP == NULL)
   {
     error("Couldn't create %s: %s\n", output_filename, strerror(errno));
     return;
   }

  fprintf(FP, "# gnuplot data for %s by %s\n", output_filename, PACKAGE_NAME);
  fprintf(FP, "# t(us) size(bytes) duration(us) bitrate(bytes/s)\n");

#if 0
  /* output each frame */
  uint64_t t = 0;
  uint64_t t_in_us, d_in_us;

  for (i=0; i<stream->packets->len; i++)
   {
     p = &g_array_index(stream->packets, struct bt_packet, i);

     t += p->duration;
     t_in_us = (double)1000000 * t * stream->time_base.num / stream->time_base.den;
     d_in_us = (double)1000000 * p->duration * stream->time_base.num / stream->time_base.den;

     fprintf(FP, "%lld %d %lld %d\n", t_in_us, p->size, d_in_us, p->bitrate);
   }
#else
  /* output one information per second */
  uint64_t t = 0;
  uint64_t t_in_us;
  uint64_t sec_size = 0;
  uint64_t sec_duration = 0;
  uint64_t sec_duration_in_us;
  unsigned int bitrate;
  for (i=0; i<stream->packets->len; i++)
   {
     p = &g_array_index(stream->packets, struct bt_packet, i);

     t += p->duration;
     sec_duration += p->duration;
     sec_size += p->size;

     sec_duration_in_us = 1000000ULL * sec_duration * stream->time_base.num / stream->time_base.den;
     if (sec_duration_in_us >= 1000000)
      {
	bitrate = sec_size * 1000000UL / sec_duration_in_us;

	t_in_us = 1000000ULL * t * stream->time_base.num / stream->time_base.den;
	fprintf(FP, "%"PRIu64" %"PRIu64" %"PRIu64" %u\n", t_in_us, sec_size, sec_duration_in_us, bitrate);

	sec_duration = 0;
	sec_size = 0;
      }

   }

  if (sec_duration)
   {
     bitrate = sec_size * 1000000UL / sec_duration_in_us;
     t_in_us = 1000000ULL * t * stream->time_base.num / stream->time_base.den;
     sec_duration_in_us = 1000000ULL * sec_duration * stream->time_base.num / stream->time_base.den;
     fprintf(FP, "%"PRIu64" %"PRIu64" %"PRIu64" %u\n", t_in_us, sec_size, sec_duration_in_us, bitrate);
   }

#endif

  fclose(FP);
}

static void
output_bitrate_gnuplot(struct video_stats *stats, const char *prefix)
{
  FILE *FP;
  char *output_filename, *datname;
  char dstring[G_ASCII_DTOSTR_BUF_SIZE];
  const char *sep = "";

  output_filename = g_strdup_printf("%s.gnuplot", prefix);

  FP = fopen(output_filename, "w");
  if (FP == NULL)
   {
     error("Couldn't create %s: %s\n", output_filename, strerror(errno));
     g_free(output_filename);
     return;
   }

  fprintf(FP, "# gnuplot script by %s\n", PACKAGE_NAME);
  fprintf(FP, "set term png size 1920, 1200\n");
  fprintf(FP, "set output '%s.png'\n", prefix);
  fprintf(FP, "set title \'%s\'\n", prefix);
  fprintf(FP, "set grid\n");
  fprintf(FP, "set xlabel \"time\"\n");
  fprintf(FP, "set ylabel \"%s\"\n", "bitrate (Kbits/s)");
  if (stats->audio)
    fprintf(FP, "audio_avg = %s\n", g_ascii_dtostr(dstring, sizeof(dstring), stream_get_avg_bitrate(stats->audio)*8/1000));
  if (stats->video)
    fprintf(FP, "video_avg = %s\n", g_ascii_dtostr(dstring, sizeof(dstring), stream_get_avg_bitrate(stats->video)*8/1000));


  fprintf(FP, "set ytics 0,10\n");
  fprintf(FP, "set xdata time\n");
  fprintf(FP, "set timefmt \"%%M:%%S\"\n");
  fprintf(FP, "plot ");

  if (stats->audio)
   {
     datname = g_strdup_printf("%s_audio.dat", prefix);
     fprintf(FP, "'%s' using ($1/1000000):($4*8/1000) with step title 'audio', ", datname);
     stream_output_bitrate_data(stats->audio, datname);
     fprintf(FP, "audio_avg title 'audio avg (%s)' " , network_size_to_string("", stream_get_avg_bitrate(stats->audio), "/s") );
     free(datname);
     sep = ",";
   }
  if (stats->video)
   {
     datname = g_strdup_printf("%s_video.dat", prefix);
     fprintf(FP, "%s '%s' using ($1/1000000):($4*8/1000) with step title 'video', ", sep, datname);
     stream_output_bitrate_data(stats->video, datname);
     free(datname);
     fprintf(FP, "video_avg title 'video avg (%s)' " , network_size_to_string("", stream_get_avg_bitrate(stats->video), "/s") );
     sep = ",";
   }
  fprintf(FP, "\n");
  fprintf(FP, "\n");

  fclose(FP);
  g_free(output_filename);
}



