/*
 * gtktube: a gtk frontend for youtube like services
 * Copyright (C) 2009 Luc Saillard <luc@saillard.org>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */
#include <config.h>

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <glib.h>
#include <glib/gi18n.h>

#include "utils.h"

void
die(const char *fmt, ... )
{
  char buf[1024];
  va_list ap;
  va_start (ap, fmt);
  snprintf (buf, sizeof(buf), "%s", fmt);
  vfprintf (stderr, buf, ap);
  exit(2);
}

void
die_with_errno(int errno, const char *fmt, ... )
{
  char buf[1024];
  va_list ap;

  va_start(ap, fmt);
  snprintf(buf, sizeof(buf), "%s (%s)\n", fmt, strerror(errno));
  vfprintf(stderr, buf, ap);
  exit(2);
}



void
error(const char *fmt, ... )
{
  char buf[1024];
  va_list ap;
  va_start(ap, fmt);
  snprintf(buf, sizeof(buf), "%s", fmt);
  vfprintf(stderr, buf, ap);
  va_end(ap);
}

#if ENABLE_DEBUG
void
real_debug(const char *fmt, ... )
{
  char buf[1024];
  va_list ap;
  va_start(ap, fmt);
  snprintf(buf, sizeof(buf), "%s", fmt);
  vfprintf(stderr, buf, ap);
  fflush(stderr);
  va_end(ap);
}
#endif

#if ENABLE_DEBUG_TRACE
void
real_trace(const char *fname, const char *fmt, ... )
{
  char buf[512];
  va_list ap;
  va_start(ap, fmt);
  snprintf(buf, sizeof(buf), "<%s> %s", fname, fmt);
  vfprintf(stderr, buf, ap);
  fflush(stderr);
  va_end(ap);
}
#endif






char *
size_to_string(const char *prefix, unsigned long long  size)
{
  if (size >= (1*TiBYTES))
    return g_strdup_printf(_("%s%.2f TiB"), prefix?prefix:"", (gdouble)size/TiBYTES);
  else if (size >= (1*GiBYTES))
    return g_strdup_printf(_("%s%.2f GiB"), prefix?prefix:"", (gdouble)size/GiBYTES);
  else if (size >= (1*MiBYTES))
    return g_strdup_printf(_("%s%.2f MiB"), prefix?prefix:"", (gdouble)size/MiBYTES);
  else if (size >= (1*KiBYTES))
    return g_strdup_printf(_("%s%.2f KiB"), prefix?prefix:"", (gdouble)size/KiBYTES);
  else
    return g_strdup_printf("%s%lld B", prefix?prefix:"", size);
}

char *
network_size_to_string(const char *prefix, unsigned long long size, const char *suffix)
{
  if (prefix == NULL)
    prefix = "";
  if (suffix == NULL)
    suffix = "";
  size = size*8;
  if (size >= (1*TBYTES))
    return g_strdup_printf(_("%s%.3f Tbits%s"), prefix, (gdouble)size/TBYTES, suffix);
  else if (size >= (1*GBYTES))
    return g_strdup_printf(_("%s%.3f Gbits%s"), prefix, (gdouble)size/GBYTES, suffix);
  else if (size >= (1*MBYTES))
    return g_strdup_printf(_("%s%.3f Mbits%s"), prefix, (gdouble)size/MBYTES, suffix);
  else if (size >= (1*KBYTES))
    return g_strdup_printf(_("%s%.2f Kbits%s"), prefix, (gdouble)size/KBYTES, suffix);
  else
    return g_strdup_printf("%s%lldbits%s", prefix, size, suffix);
}

char *
time_us_to_string(unsigned long long time)
{
  unsigned int r, h, m, s, ms, us;


  h = time / (60*60*1000000); r = time % (60*60*1000000);
  m = r / (60*1000000); r = r % (60*1000000);
  s = r / 1000000; r = r % 1000000;
  ms = r / 1000; r = r % 1000;
  us = r % 1000;

  if (h)
    return g_strdup_printf("%dh:%dm:%ds.%dms.%dus", h, m, s , ms, us);
  if (m)
    return g_strdup_printf("%dm:%ds.%dms.%dus", m, s , ms, us);
  if (s)
    return g_strdup_printf("%ds.%dms.%dus", s , ms, us);
  if (ms)
    return g_strdup_printf("%dms.%dus", ms, us);

  return g_strdup_printf("%dus", us);
}


char *
regex_match_simple(const char *text, const char *regexp)
{
  GRegex *regex;
  GMatchInfo *match_info;
  gchar *word = NULL;

  regex = g_regex_new(regexp, 0, 0, NULL);
  g_regex_match(regex, text, 0, &match_info);
  if (g_match_info_matches(match_info))
    word = g_match_info_fetch(match_info, 1);
  g_match_info_free(match_info);
  g_regex_unref(regex);

  return word;
}






